#!/usr/bin/env python

import time
import threading

from dmObject import DmObject
from dm.common.utility.dictUtility import DictUtility
from dm.common.utility.timeUtility import TimeUtility

class UploadInfo(DmObject):

    DEFAULT_KEY_LIST = [ 'id', 'experimentName', 'dataDirectory', 'status', 'nProcessedFiles', 'nProcessingErrors', 'nFiles', 'percentageComplete', 'startTime', 'endTime', 'runTime', 'startTimestamp', 'endTimestamp' ]

    def __init__(self, dict={}):
        DmObject.__init__(self, dict)
        self.lock = threading.RLock()

    def fileProcessed(self, filePath, processingEndTime):
        self.lock.acquire()
        try:
            self['nProcessedFiles'] = self.get('nProcessedFiles', 0) + 1
            lastFileProcessingEndTime = self.get('lastFileProcessingEndTime', 0)
            if processingEndTime is not None and processingEndTime > lastFileProcessingEndTime:
                self['lastFileProcessed'] = filePath
                self['lastFileProcessingEndTime'] = processingEndTime
        finally:
            self.lock.release()

    def fileProcessingError(self, filePath, processingError, processingEndTime):
        self.lock.acquire()
        try:
            processingErrors = self.get('processingErrors', {})
            processingErrors[filePath] = processingError
            self['processingErrors'] = processingErrors
            self['nProcessingErrors'] = len(processingErrors)
            lastFileProcessingEndTime = self.get('lastFileProcessingEndTime', 0)
            if processingEndTime is not None and processingEndTime > lastFileProcessingEndTime:
                self['lastFileProcessingEndTime'] = processingEndTime
        finally:
            self.lock.release()

    def updateStatus(self):
        now = time.time()
        uploadStatus = self.get('status', 'running')
        if uploadStatus == 'done':
            return 
        nFiles = self.get('nFiles', 0)
        nProcessedFiles = self.get('nProcessedFiles', 0)
        nProcessingErrors = self.get('nProcessingErrors', 0)
        processingErrors = self.get('processingErrors', {})
        nCompletedFiles = nProcessedFiles+nProcessingErrors

        startTime = self.get('startTime')
        if startTime:
            runTime = now - startTime
            self['runTime'] = runTime

        if nCompletedFiles == nFiles:
            uploadStatus = 'done'
            endTime = self.get('lastFileProcessingEndTime', now)
            self['endTime'] = endTime
            self['endTimestamp'] = TimeUtility.formatLocalTimestamp(endTime)
            if startTime:
                runTime = endTime - startTime
                self['runTime'] = runTime
        self['status'] = uploadStatus

        percentageComplete = 100.0
        percentageProcessed = 100.0
        percentageProcessingErrors = 0.0
        if nFiles > 0:
             percentageComplete = float(nCompletedFiles)/float(nFiles)*100.0
             percentageProcessed = float(nProcessedFiles)/float(nFiles)*100.0
             percentageProcessingErrors = float(nProcessingErrors)/float(nFiles)*100.0
        self['percentageComplete'] = '%.2f' % percentageComplete
        self['percentageProcessed'] = '%.2f' % percentageProcessed
        self['percentageProcessingErrors'] = '%.2f' % percentageProcessingErrors


